/*
	Chickadee(tm)
	CPU functions
		DMA channels
		Interrupt controller
		Power save unit
		Counter/timers
*/

#pragma inline

#include "stdinc.h"
#include "chickade.h"


/****************************************************************************
IRQinit():  programs IRQ channel for use

	arguments:  the vector #, ptr to interrupt handler function, priority,
		trigger spec
	returns:  nothing
	resources used:  80C188 interrupt (and possibly the DMA) controller(s)
	comments:  DOES NOT UNMASK THE INTERRUPT
		vector #:  0 - 255
		priority:  0=highest, 7 = lowest
		trigger:   0=edge, non-zero=level (does not apply to DMA interrupts
			but argument must be passed anyway)
	verified:  7/9/96 all
****************************************************************************/
void IRQinit (UBYTE vecnum, void interrupt (*vec)(), UBYTE priority,
	BOOLEAN trigger) {
	UWORD temp;

	disable ();
	setvect (vecnum, vec);
	enable ();
	switch (vecnum) {
		case INT0NUM:
		case INT1NUM:
		case INT2NUM:
		case INT3NUM:
			/* calculate interrupt control reg offset */
			temp = (vecnum - INT0NUM) * 2;
			/* program trigger and priority and leave masked */
			outport (I0CON + temp, (trigger ? 0x0010 : 0)
				| ((UWORD)priority & 0x0007) | 0x0008);
			break;
		case DMA0NUM:
		case DMA1NUM:
			/* calculate DMA control reg offset */
			temp = (vecnum - DMA0NUM) * 16;
			/* set up DMA */
			outport (D0SRCL + temp, 0x780);  /* source ptr = 0x780 i/o dummy read */
			outport (D0SRCH + temp, 0);
			outport (D0DSTL + temp, 0x780);  /* dest ptr = 0x780 i/o dummy write */
			outport (D0DSTH + temp, 0);
			/* use DMA channel for interrupts by programming it with a transfer count
				of 1, and to interrupt on terminal count */
			outport (D0TC + temp, 1);        /* transfer count = 1 */
			outport (D0CON + temp, 0x0346);    /* ctrl reg = i/o, i/o, no incr, tc, intr,
											src synch, byte xfr */
			/* calculate DMA interrupt reg offset */
			temp = (vecnum - DMA0NUM) * 2;
			/* program interrupt priority and leave masked */
			outport (DMA0CON + temp, ((UWORD)priority & 0x0007) | 0x0008);
			break;
		default:
			break;
	}
}


void unmask (int intnum) {
	outport (IMASK, inport (IMASK) & ~(1 << (intnum - 8)));
}


void mask (int intnum) {
/*	MASK usage:  MASK(x); where x is the interrupt number (8-15) */
	outport (IMASK, inport (IMASK) | (1 << (intnum - 8)));
}


/*
	CHKdelay:  about 1.875 us / iteration for 8 MHz CPU clock (80C188)
		args:  # of loop iterations to delay (1-65535)
		notes:  may vary due to wait states, etc.  May require tweaking for
			your exact setup.  An argument of 0 causes 65536 iterations.
			Interrupts may cause delays to be longer than specified.
*/
void CHKdelay (UWORD i) {
asm	{
	push	cx
	mov		cx, i
	}
CHKdelay_1:
asm	{
	loop	CHKdelay_1 	/* 16 clocks / iteration, 6 clocks for fall-thru */
	pop		cx
	}
}
